package com.jivosite.jivosdk_plugin

import android.content.Context

import android.app.Activity
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import android.content.Intent
import android.os.Handler
import android.os.Looper
import com.jivosite.sdk.ui.chat.JivoChatActivity
import com.jivosite.sdk.Jivo
import timber.log.Timber
import com.jivosite.sdk.support.builders.ContactInfo
import com.jivosite.sdk.model.pojo.CustomData
import com.jivosite.sdk.model.repository.history.*
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodChannel.Result


/** JivoPlatform */
class JivoPlugin : FlutterPlugin, ActivityAware, MethodCallHandler {

    private var applicationContext: Context? = null
    private lateinit var channel: MethodChannel
    private var mainActivity: Activity? = null
    private var newMessageListener: NewMessageListener? = null
    private val handler = Handler(Looper.getMainLooper())

    override fun onAttachedToEngine(flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        Timber.plant(Timber.DebugTree())
        applicationContext = flutterPluginBinding.applicationContext.also {
            Jivo.init(it)
        }
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "jivosdk_plugin")
        channel.setMethodCallHandler(this)
        Jivo.enableLogging()

        Jivo.addNewMessageListener(object : NewMessageListener {
            override fun onNewMessage(hasUnread: Boolean) {
                handler.post {
                    channel.invokeMethod("session:updateUnreadCounter(number)", if (hasUnread) 1 else 0)
                }
            }
        }.apply {
            newMessageListener = this
        })
    }


    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        mainActivity = binding.activity
    }

    override fun onDetachedFromActivityForConfigChanges() {
        mainActivity = null
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        mainActivity = binding.activity
    }

    override fun onDetachedFromActivity() {
        mainActivity = null
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            "session:setup" -> {
                val data = call.arguments as Map<String, String>
                val widgetId = data["channel_id"] ?: ""
                val userToken = data["userToken"] ?: ""
                if (widgetId.contains('/')) {
                    val list = widgetId.split('/')
                    Jivo.setData(widgetId = list.last(), userToken = userToken, host = list.first())
                } else {
                    Jivo.setData(widgetId = widgetId, userToken = userToken)
                }
                result.success(null)
            }

            "notification:setPushToken" -> {
                val token = call.arguments as String
                if (token.isNotBlank()) {
                    Jivo.updatePushToken(call.arguments as String)
                }
            }

            "session:setContactInfo" -> {
                val contactInfo = call.arguments as Map<String, String>
                Jivo.setContactInfo(ContactInfo.contactInfo {
                    name = contactInfo["name"]
                    email = contactInfo["email"] ?: ""
                    phone = contactInfo["phone"] ?: ""
                    description = contactInfo["brief"] ?: ""
                })
                result.success(null)
            }

            "session:setCustomData" -> {
                val fields = call.arguments as List<Map<String, String>>
                Jivo.setCustomData(fields.map {
                    CustomData(
                        content = it["content"] ?: "", title = it["title"] ?: "", link = it["link"] ?: "", key = it["key"] ?: ""
                    )
                })
                result.success(null)
            }

            "display:present" -> {
                Intent(mainActivity, JivoChatActivity::class.java).apply {
                    if (mainActivity?.packageManager?.let { resolveActivity(it) } != null) {
                        flags = Intent.FLAG_ACTIVITY_NEW_TASK
                        applicationContext?.startActivity(this)
                    }
                }
                result.success(null)
            }

            "session:clear" -> {
                Jivo.clear()
                result.success(null)
            }

            else -> result.notImplemented()
        }
    }

    override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
        applicationContext = null
        newMessageListener = null
    }
}
